#include <QGuiApplication>
#include <QQmlApplicationEngine>

#include "PrefixTickLabelsModel.h"

int main(int argc, char *argv[])
{
    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);

    QGuiApplication app(argc, argv);

    QQmlApplicationEngine engine;
    qmlRegisterType<PrefixTickLabelsModel>("Demo", 1, 0, "PrefixTickLabelsModel");

    const QUrl url(QStringLiteral("qrc:/demo.qml"));
    QObject::connect(&engine, &QQmlApplicationEngine::objectCreated,
        &app, [url](QObject *obj, const QUrl &objUrl) {
            if (!obj && url == objUrl)
                QCoreApplication::exit(-1);
        }, Qt::QueuedConnection);
    engine.load(url);

    return app.exec();
}
